#version 330

in vec2 TexCoord;

out vec4 fragColor;
out float gl_FragDepth;

uniform bool renderHalos;

uniform float stripWidth;

uniform float minRatio;
uniform float maxRatio;

uniform int depth_cueing;
uniform int falloff;

uniform mat4 projection;

uniform sampler2D texture1;


void main()
{

    float vTexCoord = TexCoord.y;

    float dmax = stripWidth;

    // Depth calculations are done in View-Space, because Depth is non-linear in Eye-Space.
    // Values at the near-plane are 0, values at the far-plane are 1. Due to non-linearity, most
    // Depth values range from 0.9 to 1.0. The non-linearity also leads to artefacts when changing
    // the camera position and makes depth-cueing very hard to do.
    // Code-Snippet from https://community.khronos.org/t/gl-fragcoord-z/54092/9
    float eyeSpaceDepth = projection[3].z/(gl_FragCoord.z * -2.0 + 1.0 - projection[2].z);

    // ratio between line-width and strip-width for closest fragments
    // float maxLineWidth = 0.5;
    // float falloff = 3;

    float lineWidthRatio = 0.0;

    if(depth_cueing == 1) {
        lineWidthRatio = (1/(-eyeSpaceDepth * falloff));
    }
    else {
        lineWidthRatio = (1 + (eyeSpaceDepth * 0.1) * falloff);
    }

    // limit line width to meaningful values
    // float maxRatio = 0.75;
    // float minRatio = 0.1;
    lineWidthRatio = min(max(lineWidthRatio,minRatio),maxRatio);

    float lineWidth = lineWidthRatio * stripWidth;

    // s is the distance between the fragment and the center of the line
    float s = stripWidth * abs(vTexCoord-0.5);

    if(s < (lineWidth/(2))){
        fragColor = vec4( 0.0, 0.0, 0.0, 1.0 );
        gl_FragDepth = gl_FragCoord.z;
    }
    else{

        if(renderHalos == false){
            discard;
        }

        fragColor = vec4( 1.0, 1.0, 1.0, 1.0 );
		
        // calculate the depth-offset for halo as suggested by Everts et al. (but in Eye-Space)
        float eyeSpaceDepth_offset = eyeSpaceDepth - dmax*(2*abs(vTexCoord-0.5));

        // Transform from Eye-Space to NDC
        // Code-Snippet from https://community.khronos.org/t/ndc-depth-to-pre-perspective-divide-depth/60758
        float z_ndc = (eyeSpaceDepth_offset*projection[2].z + projection[3].z) / -eyeSpaceDepth_offset;

        // Transform from NDC to Screen-Space by mapping [-1,1] to [0,1]
        float screenSpaceDepth = (z_ndc+1.0)/2.0;

        gl_FragDepth = screenSpaceDepth;

        // gl_FragDepth =  gl_FragCoord.z + dmax*(2*abs(vTexCoord-0.5));

    }

    //Apply tapering
    if (texture(texture1, TexCoord) == vec4( 1.0, 1.0, 1.0, 1.0 )){
        discard;
    }

}
