#version 330

in float vTexCoord;
in float uTexCoord;

out vec4 fragColor;
out float gl_FragDepth;

uniform bool renderHalos;
uniform float coneWidth;
uniform float radius;
uniform mat4 projection;

void main()
{

    float dmax = coneWidth;

    // Depth calculations are done in View-Space, because Depth is non-linear in Eye-Space.
    // Values at the near-plane are 0, values at the far-plane are 1. Due to non-linearity, most
    // Depth values range from 0.9 to 1.0. The non-linearity also leads to artefacts when changing
    // the camera position and makes depth-cueing very hard to do.
    // Code-Snippet from https://community.khronos.org/t/gl-fragcoord-z/54092/9
    float eyeSpaceDepth = projection[3].z/(gl_FragCoord.z * -2.0 + 1.0 - projection[2].z);


    // use circle-equation to draw circle onto the billboard
    if(pow(uTexCoord,2.0) + pow(vTexCoord,2.0) <= pow(radius,2.0)){

        fragColor = vec4( 0.0, 0.0, 0.0, 1.0 );
        gl_FragDepth = gl_FragCoord.z;

    }
    else{

        if(renderHalos == false){
            discard;
        }

        fragColor = vec4( 1.0, 1.0, 1.0, 1.0 );
		
        // calculate the depth-offset for halo as suggested by Everts et al. (but in Eye-Space)
        // for point-data, we use the euclidean distance from the center to generate a cone
        float eyeSpaceDepth_offset = eyeSpaceDepth - dmax*(sqrt(pow(vTexCoord,2.0)+pow(uTexCoord,2.0)));

        // Transform from Eye-Space to NDC
        // Code-Snippet from https://community.khronos.org/t/ndc-depth-to-pre-perspective-divide-depth/60758
        float z_ndc = (eyeSpaceDepth_offset*projection[2].z + projection[3].z) / -eyeSpaceDepth_offset;

        // Transform from NDC to Screen-Space by mapping [-1,1] to [0,1]
        float screenSpaceDepth = (z_ndc+1.0)/2.0;

        gl_FragDepth = screenSpaceDepth;

    }

}
