#ifndef CAMERA_H
#define CAMERA_H

#include <glm.hpp>
#include <gtc/matrix_transform.hpp>

#include <vector>

enum Camera_Movement {
    FORWARD,
    BACKWARD,
    LEFT,
    RIGHT
};


class Camera
{
public:

    glm::vec3 camera_position;
    glm::vec3 world_up;
    glm::vec3 up;
    glm::vec3 front;
    glm::vec3 right;

    float yaw;
    float pitch;

    float camera_speed;
    float mouse_speed;

    /**
     * @brief Constructor (set initial position of camera)
     * @param position
     */
    Camera(glm::vec3 position = glm::vec3(0.0f, 0.0f, 0.0f))
    {
        camera_position = position;
        world_up = glm::vec3(0.0f, 1.0f, 0.0f);
        front = glm::vec3(0.0f, 0.0f, -1.0f);
        camera_speed = 2.5f;
        mouse_speed = 0.1f;
        yaw = -90.0f;
        pitch = 0.0f;

        update_camera();
    }

    /**
     * @brief get current view matrix
     * @return
     */
    glm::mat4 GetViewMatrix()
    {
        return glm::lookAt(camera_position, camera_position + front, up);
    }

    /**
     * @brief move camera in given direction (frame independent)
     * @param direction
     * @param delta_time
     */
    void move_camera(Camera_Movement direction, float delta_time)
    {
        float velocity = camera_speed * delta_time;
        if (direction == FORWARD)
            camera_position += front * velocity;
        if (direction == BACKWARD)
            camera_position -= front * velocity;
        if (direction == LEFT)
            camera_position -= right * velocity;
        if (direction == RIGHT)
            camera_position += right * velocity;
    }

    /**
     * @brief rotate camera
     * @param x_offset
     * @param y_offset
     */
    void rotate_camera(float x_offset, float y_offset)
    {
        x_offset *= mouse_speed;
        y_offset *= mouse_speed;

        yaw += x_offset;
        pitch += y_offset;

        update_camera();
    }


    /**
     * @brief update camera
     */
    void update_camera()
    {
        glm::vec3 new_front = glm::vec3(cos(glm::radians(yaw)) * cos(glm::radians(pitch)), sin(glm::radians(pitch)), sin(glm::radians(yaw)) * cos(glm::radians(pitch)));
        front = glm::normalize(new_front);

        right = glm::normalize(glm::cross(front, world_up));
        up = glm::normalize(glm::cross(right, front));
    }
};
#endif
